local addonName, E = ...

local scaler = CreateFrame("Frame")
scaler:RegisterEvent("VARIABLES_LOADED")
scaler:RegisterEvent("UI_SCALE_CHANGED")
scaler:SetScript("OnEvent", function(self, event)
	if not InCombatLockdown() then
		local scale = 768/string.match(({GetScreenResolutions()})[GetCurrentResolution()], "%d+x(%d+)")
		if scale < .64 then
			UIParent:SetScale(scale)
		else
			self:UnregisterEvent("UI_SCALE_CHANGED")
			SetCVar("uiScale", scale)
		end
	else
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	end

	if event == "PLAYER_REGEN_ENABLED" then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	end
end)

local hidenframe = CreateFrame("Frame", "AleaUIHidenFrame")
hidenframe.unit = ""
hidenframe.buffsOnTop = false
hidenframe.auraRows = 0
hidenframe:Hide()
--/script DEFAULT_CHAT_FRAME:AddMessage( GetMouseFocus():GetName() );
--[==[
local coordx1 = 8
local coordx2 = 23
local coordy1 = 10
local coordy2 = 22

return '|TInterface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up:0:0:0:0:32:32:'..coordx1..':'..coordx2..':'..coordy1..':'..coordy2..'|t'
]==]

local function SetColorTexture(obj, r,g,b,a)
	if E.IsLegion then
		obj:SetColorTexture(r,g,b,a)
	else
		obj:SetTexture(r,g,b,a)
	end
end

local function SetOutside(obj, anchor, xOffset, yOffset)
	xOffset = xOffset or 1
	yOffset = yOffset or 1
	anchor = anchor or obj:GetParent()
	
	if obj:GetPoint() then
		obj:ClearAllPoints()
	end
	
	obj:SetPoint('TOPLEFT', anchor, 'TOPLEFT', -xOffset, yOffset)
	obj:SetPoint('BOTTOMRIGHT', anchor, 'BOTTOMRIGHT', xOffset, -yOffset)
end

local function SetInside(obj, anchor, xOffset, yOffset)
	xOffset = xOffset or 1
	yOffset = yOffset or 1
	anchor = anchor or obj:GetParent()
	
	if obj:GetPoint() then
		obj:ClearAllPoints()
	end
	
	obj:SetPoint('TOPLEFT', anchor, 'TOPLEFT', xOffset, -yOffset)
	obj:SetPoint('BOTTOMRIGHT', anchor, 'BOTTOMRIGHT', -xOffset, yOffset)
end

local function RePointVertival(obj, value)
	local a1, a2, a3, a4, a5 = obj:GetPoint()
	obj:SetPoint(a1, a2, a3, a4, a5+value)
end

local function RePointHorizontal(obj, value)
	local a1, a2, a3, a4, a5 = obj:GetPoint()
	obj:SetPoint(a1, a2, a3, a4+value, a5)
end

local function SetIgnoreFramePositionManager(obj, value)	
	if value then
		obj.ignoreFramePositionManager = true
		
		if obj.SetAttribute then
			obj:SetAttribute("ignoreFramePositionManager", true)
		end
	else
		obj.ignoreFramePositionManager = nil
		
		if obj.SetAttribute then
			obj:SetAttribute("ignoreFramePositionManager", false)
		end
	end
end

local function StyleButton(button, noHover, noPushed, noChecked)
	if button.SetHighlightTexture and not button.hover and not noHover then
		local hover = button:CreateTexture()
		hover:SetColorTexture(1, 1, 1, 0.3)
		hover:SetInside()
		button.hover = hover
		button:SetHighlightTexture(hover)
	end

	if button.SetPushedTexture and not button.pushed and not noPushed then
		local pushed = button:CreateTexture()
		pushed:SetColorTexture(0.9, 0.8, 0.1, 0.3)
		pushed:SetInside()
		button.pushed = pushed
		button:SetPushedTexture(pushed)
	end

	if button.SetCheckedTexture and not button.checked and not noChecked then
		local checked = button:CreateTexture()
		checked:SetColorTexture(1, 1, 1)
		checked:SetInside()
		checked:SetAlpha(0.3)
		button.checked = checked
		button:SetCheckedTexture(checked)
	end

	local cooldown = button:GetName() and _G[button:GetName().."Cooldown"]
	if cooldown then
		cooldown:ClearAllPoints()
		cooldown:SetInside()
		cooldown:SetDrawEdge(false)
		cooldown:SetSwipeColor(0, 0, 0, 1)
	end
end


local function Kill(object)
	if object.UnregisterAllEvents then
		object:UnregisterAllEvents()
		object:SetParent(hidenframe)
	else
		object.Show = object.Hide
	end
	
	object:Hide()
end

local EnableMinimapMoving, DisableMinimapMoving
local onClick, onMouseUp, onMouseDown, onDragStart, onDragStop, updatePosition
local minimapShapes = {
	["ROUND"] = {true, true, true, true},
	["SQUARE"] = {false, false, false, false},
	["CORNER-TOPLEFT"] = {false, false, false, true},
	["CORNER-TOPRIGHT"] = {false, false, true, false},
	["CORNER-BOTTOMLEFT"] = {false, true, false, false},
	["CORNER-BOTTOMRIGHT"] = {true, false, false, false},
	["SIDE-LEFT"] = {false, true, false, true},
	["SIDE-RIGHT"] = {true, false, true, false},
	["SIDE-TOP"] = {false, false, true, true},
	["SIDE-BOTTOM"] = {true, true, false, false},
	["TRICORNER-TOPLEFT"] = {false, true, true, true},
	["TRICORNER-TOPRIGHT"] = {true, false, true, true},
	["TRICORNER-BOTTOMLEFT"] = {true, true, false, true},
	["TRICORNER-BOTTOMRIGHT"] = {true, true, true, false},
}

function updatePosition(button)
	local angle = math.rad(button.settings and button.settings.minimapPos or button.minimapPos or 225)
	local x, y, q = math.cos(angle), math.sin(angle), 1
	if x < 0 then q = q + 1 end
	if y > 0 then q = q + 2 end
	local minimapShape = GetMinimapShape and GetMinimapShape() or "ROUND"
	local quadTable = minimapShapes[minimapShape]
	if quadTable[q] then
		x, y = x*80, y*80
	else
		local diagRadius = 103.13708498985 --math.sqrt(2*(80)^2)-10
		x = math.max(-80, math.min(x*diagRadius, 80))
		y = math.max(-80, math.min(y*diagRadius, 80))
	end

	button:SetPoint("CENTER", Minimap, "CENTER", x, y)
end

local function onUpdate(me)
	local self = me.parent

	local mx, my = Minimap:GetCenter()
	local px, py = GetCursorPosition()
	local scale = Minimap:GetEffectiveScale()
	
	px, py = px /scale, py / scale
	if self.settings then
		self.settings.minimapPos = math.deg(math.atan2(py - my, px - mx)) % 360
	else
		self.minimapPos = math.deg(math.atan2(py - my, px - mx)) % 360
	end	
	updatePosition(self)
end
	
function onDragStart(self)
	self:LockHighlight()
	self._onupdate:SetScript("OnUpdate", onUpdate)
	GameTooltip:Hide()
end

function onDragStop(self)
	self._onupdate:SetScript("OnUpdate", nil)
	self:UnlockHighlight()
end
	
function EnableMinimapMoving(obj, settings)
	if not AleaUI.db.Frames[settings] then
		AleaUI.db.Frames[settings] = {}
	end
	obj._onupdate = CreateFrame("Frame", nil, obj)
	obj._onupdate.parent = obj
	obj.minimapButtonFadeOut = true
	obj:RegisterForDrag("LeftButton")
	obj.settings = AleaUI.db.Frames[settings]
	obj:HookScript("OnDragStop", onDragStop)
	obj:HookScript("OnDragStart", onDragStart)
	obj:ClearAllPoints()
	
	updatePosition(obj)
end

function DisableMinimapMoving(obj)
	
	obj.settings = nil

	obj:SetScript("OnDragStart", nil)
	obj:SetScript("OnDragStop", nil)	
end

local function StripTextures(object, kill)
	for i=1, object:GetNumRegions() do
		local region = select(i, object:GetRegions())
		if region and region:GetObjectType() == "Texture" then
		
			if kill and type(kill) == 'boolean' then
				region:Kill()
			elseif region:GetDrawLayer() == kill then
				region:SetTexture(nil)
			elseif kill and type(kill) == 'string' and region:GetTexture() ~= kill then
				region:SetTexture(nil)
			else
				region:SetTexture(nil)
			end
		end
	end
end

local function StripTextures2(object, kill)
	for i=1, object:GetNumRegions() do
		local region = select(i, object:GetRegions())
		if region and region:GetObjectType() == "Texture" then
			if kill and region:GetTexture() == kill then
				region:SetTexture(nil)
			end
		end
	end
end

local function SetResize(object, width, height)
	local o1, o2 = object:GetSize()
	
	if height then
		object:SetSize(o1+width, o2+height)
	else
		object:SetSize(o1+width, o2+width)
	end
end

local function addapi(object)
	local mt = getmetatable(object).__index
	if not object.SetOutside then mt.SetOutside = SetOutside end
	if not object.SetInside then mt.SetInside = SetInside end
	if not object.Kill then mt.Kill = Kill end
	if not object.StripTextures then mt.StripTextures = StripTextures end
	if not object.StripTextures2 then mt.StripTextures2 = StripTextures2 end
	if not object.RePointVertival then mt.RePointVertival = RePointVertival end
	if not object.RePointHorizontal then mt.RePointHorizontal = RePointHorizontal end
	if not object.SetIgnoreFramePositionManager then mt.SetIgnoreFramePositionManager = SetIgnoreFramePositionManager end
	if not object.SetResize then mt.SetResize = SetResize end
	if not object.EnableMinimapMoving then mt.EnableMinimapMoving = EnableMinimapMoving end
	if not object.DisableMinimapMoving then mt.DisableMinimapMoving = DisableMinimapMoving end
	if not object.StyleButton then mt.StyleButton = StyleButton end
	if not object.SetColorTexture and object.SetTexture then mt.SetColorTexture = SetColorTexture end
end

local handled = {["Frame"] = true}
local object = CreateFrame("Frame")
addapi(object)
addapi(object:CreateTexture())
addapi(object:CreateFontString())

object = EnumerateFrames()
while object do
	if not handled[object:GetObjectType()] then
		addapi(object)
		handled[object:GetObjectType()] = true
	end
	
	object = EnumerateFrames(object)
end

do
	local addon = ...
	local C = {}
	local addonChannel = "AleaUIVCH"
	local remindMeagain = true
	local name
	local string_match = string.match
	local format = format
	local SendAddonMessage = SendAddonMessage
	local tonumber = tonumber
	local sendmessagethottle = 20
	local versioncheck = 0

	if not IsAddonMessagePrefixRegistered(addonChannel) then
		RegisterAddonMessagePrefix(addonChannel)
	end

	function C:AddonMessage(msg, channel)
		if channel == "GUILD" and IsInGuild() then
			SendAddonMessage(addonChannel, msg, "GUILD")
		else
			local chatType = "PRINT"
			if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE) then
				chatType = "INSTANCE_CHAT"
			elseif IsInRaid(LE_PARTY_CATEGORY_HOME) then
				chatType = "RAID"
			elseif IsInGroup(LE_PARTY_CATEGORY_HOME) then
				chatType = "PARTY"
			end
				
			if chatType == "PRINT" then
				
			else
				SendAddonMessage(addonChannel, msg, chatType)
			end
		end
	end

	local function constructVersion(ver)
		local d1, d2, d3 = strsplit(".", ver)
		
		d1 = d1 or "0"
		d2 = d2 or "0"
		d3 = d3 or "0"
		
		if #d2 == 1 then
		   d2 = "00"..d2
		end
		if #d2 == 2 then
		   d2 = "0"..d2
		end
		if #d3 == 1 then
		   d3 = "00"..d3
		end
		if #d3 == 2 then
		   d3 = "0"..d3
		end
		
		return tonumber(d1..d2..d3)
	end

	local events = CreateFrame("Frame")
	events:SetScript("OnEvent", function(self, event, ...)
		self[event](self, event, ...)
	end)

	function events:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
		if prefix ~= addonChannel then return end
		if sender == name then return end
		if not remindMeagain then return end	
		local version, source = strsplit(":", message)	
		if version and source then
			local cntrV = constructVersion(version)
			local cntrmV = constructVersion(C.myVersionT)
		
			if cntrV > cntrmV then
				remindMeagain = false
				print(addon..": New version".." "..version.." ".."availible on https://mods.curse.com/addons/wow/aleaui")
			end
		end
	end

	function events:SendAddonIndo()
		if GetTime() < versioncheck then return end
		versioncheck = GetTime() + sendmessagethottle
		C:AddonMessage(format("%s:%s", C.myVersionT, C.VersionSource))
	end

	function events:SendAddonIndo2()
		if GetTime() < versioncheck then return end
		versioncheck = GetTime() + sendmessagethottle
		C:AddonMessage(format("%s:%s", C.myVersionT, C.VersionSource) , "GUILD")
	end

	events.GROUP_ROSTER_UPDATE = events.SendAddonIndo
	events.PLAYER_ENTERING_WORLD = events.SendAddonIndo2
	events.PLAYER_ENTERING_BATTLEGROUND = events.SendAddonIndo
	events.GROUP_JOINED = events.SendAddonIndo
	events.RAID_INSTANCE_WELCOME = events.SendAddonIndo
	events.ZONE_CHANGED_NEW_AREA = events.SendAddonIndo

	events.GUILD_MOTD = events.SendAddonIndo2
	events.GUILD_NEWS_UPDATE = events.SendAddonIndo2
	events.GUILD_ROSTER_UPDATE = events.SendAddonIndo2


	events:RegisterEvent("PLAYER_LOGIN")

	function events:PLAYER_LOGIN()
		local version = GetAddOnMetadata(addon, "Version") or "0"
		local version_c = version:gsub("%.", "")
		
		name = UnitName("player").."-"..GetRealmName()

		C.myVersionT = version
		C.myVersion = tonumber(version_c) or 0
		C.VersionSource = GetAddOnMetadata(addon, "VersionType") or "main"

		events:RegisterEvent("CHAT_MSG_ADDON")
		events:RegisterEvent("GROUP_ROSTER_UPDATE")
		events:RegisterEvent("PLAYER_ENTERING_WORLD")
		events:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND")
		events:RegisterEvent("GROUP_JOINED")
		events:RegisterEvent("RAID_INSTANCE_WELCOME")
		events:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		events:RegisterEvent("GUILD_MOTD")
		events:RegisterEvent("GUILD_NEWS_UPDATE")
		events:RegisterEvent("GUILD_ROSTER_UPDATE")
		
		events:SendAddonIndo()
		events:SendAddonIndo2()
	end
end

do
	local LockedCVars = {}
	local CVarUpdate = nil
	
	local event = CreateFrame('Frame')
	event:RegisterEvent('PLAYER_REGEN_ENABLED')
	event:SetScript('OnEvent', function(event)
		if(CVarUpdate) then
			for cvarName, value in pairs(LockedCVars) do
				if(GetCVar(cvarName) ~= value) then
					SetCVar(cvarName, value)
				end			
			end
			CVarUpdate = nil
		end
	end)

	local function CVAR_UPDATE(cvarName, value)
		if(LockedCVars[cvarName] and LockedCVars[cvarName] ~= value) then
			if(InCombatLockdown()) then
				CVarUpdate = true
				return
			end
			
			SetCVar(cvarName, LockedCVars[cvarName])
		end
	end

	hooksecurefunc("SetCVar", CVAR_UPDATE)
	function E:LockCVar(cvarName, value, force)
		if(GetCVar(cvarName) ~= value) or force then
			SetCVar(cvarName, value)
		end
		LockedCVars[cvarName] = value
	end
end


do
	local PADDING = 10
	local BUTTON_HEIGHT = 16
	local BUTTON_WIDTH = 135

	local function OnClick(btn)
		btn.func()

		btn:GetParent():Hide()
	end

	local function OnEnter(btn)
		btn.hoverTex:Show()
		btn:GetParent().elapsed = 0
	end

	local function OnLeave(btn)
		btn.hoverTex:Hide()
		btn:GetParent().elapsed = 0
	end

	function E:DropDown(list, frame, xOffset, yOffset)
		if not frame.buttons then
			frame.buttons = {}
			frame:SetFrameStrata("DIALOG")
			frame:SetClampedToScreen(true)
			tinsert(UISpecialFrames, frame:GetName())
			frame:Hide()
		end

		xOffset = xOffset or 0
		yOffset = yOffset or 0

		for i=1, #frame.buttons do
			frame.buttons[i]:Hide()
		end

		for i=1, #list do
			if not frame.buttons[i] then
				frame.buttons[i] = CreateFrame("Button", nil, frame)

				frame.buttons[i].hoverTex = frame.buttons[i]:CreateTexture(nil, 'OVERLAY')
				frame.buttons[i].hoverTex:SetAllPoints()
				frame.buttons[i].hoverTex:SetTexture([[Interface\QuestFrame\UI-QuestTitleHighlight]])
				frame.buttons[i].hoverTex:SetBlendMode("ADD")
				frame.buttons[i].hoverTex:Hide()

				frame.buttons[i].text = frame.buttons[i]:CreateFontString(nil, 'BORDER')
				frame.buttons[i].text:SetAllPoints()
				frame.buttons[i].text:SetFont(STANDARD_TEXT_FONT, 10, 'NONE') 
				frame.buttons[i].text:SetJustifyH("LEFT")

				frame.buttons[i]:SetScript("OnEnter", OnEnter)
				frame.buttons[i]:SetScript("OnLeave", OnLeave)
			end

			frame.buttons[i]:Show()
			frame.buttons[i]:SetHeight(BUTTON_HEIGHT)
			frame.buttons[i]:SetWidth(BUTTON_WIDTH)
			frame.buttons[i].text:SetText(list[i].text)
			frame.buttons[i].func = list[i].func
			frame.buttons[i]:SetScript("OnClick", OnClick)

			if i == 1 then
				frame.buttons[i]:SetPoint("TOPLEFT", frame, "TOPLEFT", PADDING, -PADDING)
			else
				frame.buttons[i]:SetPoint("TOPLEFT", frame.buttons[i-1], "BOTTOMLEFT")
			end
		end

		frame:SetHeight((#list * BUTTON_HEIGHT) + PADDING * 2)
		frame:SetWidth(BUTTON_WIDTH + PADDING * 2)

		local UIScale = UIParent:GetScale();
		local x, y = GetCursorPosition();
		x = x/UIScale
		y = y/UIScale
		frame:ClearAllPoints()
		frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x + xOffset, y + yOffset)
		frame:SetScript('OnUpdate', function(self, elapsed)
			frame.elapsed = ( frame.elapsed or 0 ) + elapsed
			if frame.elapsed < 3 then return end
			frame.elapsed = 0	
			ToggleFrame(self)
		end)
		frame:SetScript('OnShow', function(self)
			self.elapsed = 0
		end)
		frame:SetScript('OnHide', function(self)
			self.elapsed = 0
		end)
		
		ToggleFrame(frame)
	end
end

do
	local list = {}
	local list2 = {}
	local list3 = {}
	local list4 = {}
	
	local type = type
	local tinsert = table.insert

	function E:OnInit(func)
		if not func then return end
		if type(func) == "function" then
			tinsert(list, func)
		elseif type(func) == "string" and E[func] then
			tinsert(list, E[func])
		end
	end
	function E:InitFrames()
		for i, func in ipairs(list) do		
			func()
		end		
		wipe(list)
	end
	
	
	function E:OnInit2(func)
		if not func then return end
		if type(func) == "function" then
			tinsert(list2, func)
		elseif type(func) == "string" and E[func] then
			tinsert(list2, E[func])
		end
	end	
	function E:InitFrames2()
		for i, func in ipairs(list2) do
			func()
		end	
		wipe(list2)
	end
	
	
	function E:OnPostInit(func)
		if not func then return end
		if type(func) == "function" then
			tinsert(list3, func)
		elseif type(func) == "string" and E[func] then
			tinsert(list3, E[func])
		end
	end
	function E:InitFrames3()
		for i, func in ipairs(list3) do		
			func()
		end		
		wipe(list3)
	end
	
	function E:OnPostInit2(func)
		if not func then return end
		if type(func) == "function" then
			tinsert(list4, func)
		elseif type(func) == "string" and E[func] then
			tinsert(list4, E[func])
		end
	end
	function E:InitFrames4()
		for i, func in ipairs(list4) do		
			func()
		end		
		wipe(list4)
	end
end

do
	local modules = {}
	
	function E:Module(name)
	
		if not modules[name] then
			modules[name] = CreateFrame("Frame", "AleaUI"..name.."Module")
			modules[name]:SetScript("OnEvent", function(self, event, ...)
				if not self[event] then error('Unused event '..event) end
				self[event](self, event, ...)
			end)
			
			
			modules[name].Register = function(self, events)
				for k,v in pairs(events) do
					self:RegisterEvent(v)				
				end
			end
		end
		
		return modules[name]
	end
end

do	
	local list = {}
	local function OnAddonLoad(core, addon, func)
		if not list[addon] then list[addon] = {} end
		list[addon][#list[addon]+1] = func
	end
	
	local onaddonloaded = CreateFrame("Frame")
	onaddonloaded:RegisterEvent("ADDON_LOADED")
	onaddonloaded:SetScript("OnEvent", function(self, event, addon, ...)	
		if list[addon] then
			for i=1, #list[addon] do
				list[addon][i]()
			end
		end
	end)
	
	E.OnAddonLoad = OnAddonLoad
end