﻿local E = AleaUI
local AM = E:Module("Alerts")

local defaults = {
	
	growUp = true,
	showInCombat = true,
	texture = E.media.default_bar_texture_name2,
	fadeTime = 4,
	colors = {
		["ACH"] 			= {215/255, 174/255, 0, 1},
		["ITEM"] 			= { 61/255, 156/255, 153/255, 1},
		["LFG"] 			= { 0, 149/255, 215/255, 1},
		["MONEY"] 			= {97/255, 79/255, 0, 1},
		["CRITERIA"]		= {129/255, 110/255, 152/255, 1},
		["GARRISON"]		= {5/255, 50/255, 14/255, 1},
		["GARRISON_FOLLOW"]	= {5/255, 50/255, 14/255, 1},	
		['RESORCES'] 		= {5/255, 50/255, 14/255, 1},
		["GARRISON_BUILD"] 	= {200/255, 10/255, 10/255, 1},
		["NEW_RECIPE"]		= { 153/255, 0, 153/255, 1},
		['QUEST']   		= { 215/255, 174/255, 0, 1},
		['XP']				= { 0, 149/255, 215/255, 1},
	},
}

E.default_settings.alerts = defaults


local MAX_ACHIEVEMENT_ALERTS = 2;
local LOOT_WON_ALERT_FRAMES = {};
local MONEY_WON_ALERT_FRAMES = {};
local DELAYED_ACHIEVEMENT_ALERTS = {};
local ACHIEVEMENT_ID_INDEX = 1;
local OLD_ACHIEVEMENT_INDEX = 2;
local MAX_QUEUED_ACHIEVEMENT_TOASTS = 6;

	
local events = {
	"ACHIEVEMENT_EARNED", 
	"CRITERIA_EARNED", 
	"LFG_COMPLETION_REWARD", 
	"GUILD_CHALLENGE_COMPLETED",
	"CHALLENGE_MODE_COMPLETED",
	"LOOT_ITEM_ROLL_WON",
	"SHOW_LOOT_TOAST",
	"PET_BATTLE_CLOSE",
	"STORE_PRODUCT_DELIVERED",
	"GARRISON_BUILDING_ACTIVATABLE",
	"GARRISON_MISSION_FINISHED",
	"GARRISON_FOLLOWER_ADDED",
	"SHOW_LOOT_TOAST_UPGRADE",
	'SHOW_PVP_FACTION_LOOT_TOAST',
	
	"NEW_RECIPE_LEARNED",
	"SHOW_LOOT_TOAST_LEGENDARY_LOOTED",
	
	"SCENARIO_COMPLETED",	
	"QUEST_LOOT_RECEIVED",
	
	"GARRISON_RANDOM_MISSION_ADDED",
}	

local garrison_icons = {
	["GarrMission_MissionIcon-Combat"] = {0.39,0.51,0.13,0.25},
	
	[" "] = { 0.13,0.25,0.25,0.39 },
}

local alertsName = {
	["ACH"] 				= 'Достижениия',
	["ITEM"] 				= 'Предмет',
	["LFG"] 				= 'Поиск подземелий',
	["MONEY"] 				= 'Деньги',
	["CRITERIA"]			= 'Критерий',
	["GARRISON"]			= 'Выполнение миссии',
	["GARRISON_FOLLOW"]		= 'Новый соратник',
	["GARRISON_BUILD"] 		= 'Постройка',
	['RESORCES'] 			= 'Ресурсы',
	["NEW_RECIPE"] 			= 'Рецепт',
	['QUEST'] 	 			= 'Задания',
	['XP']					= XP,
}


local colors = {
	["ACH"] 		= {215/255, 174/255, 0, 1},
	["ITEM"] 		= { 61/255, 156/255, 153/255, 1},
	["LFG"] 		= { 0, 149/255, 215/255, 1},
	["MONEY"] 		= {97/255, 79/255, 0, 1},
	["CRITERIA"]	= {129/255, 110/255, 152/255, 1},
	["TEST"]		= {1, 1, 0, 1},
	["GARRISON"]	= {5/255, 50/255, 14/255, 1},
	["GARRISON_FOLLOW"]	= {5/255, 50/255, 14/255, 1},	
	['RESORCES'] 		= {5/255, 50/255, 14/255, 1},
	["GARRISON_BUILD"] 	= {200/255, 10/255, 10/255, 1},
	["NEW_RECIPE"]		= { 153/255, 0, 153/255, 1},
	['QUEST']   		= { 215/255, 174/255, 0, 1},
	['XP']				= { 0, 149/255, 215/255, 1},
}

local ItemQualityTable

do
	ItemQualityTable = {}
	for i = 0, 7 do
	  local r, g, b, hex = GetItemQualityColor(i);
	  ItemQualityTable[i] = { r = r, g = g, b = b, hex = hex }
	end
end

local gametooltip = CreateFrame("GameTooltip", "AleaUIAlertsGameToolTip", nil, "GameTooltipTemplate"); -- Tooltip name cannot be nil	
gametooltip:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = nil})
gametooltip:SetBackdropColor(0,0,0,0.7)
gametooltip:SetBackdropBorderColor(0,0,0,1)
gametooltip:SetScale(0.7)
gametooltip:Show();gametooltip:Hide()

local mover = CreateFrame("Frame", nil, AleaUI.UIParent)
mover:SetSize(350,20)
mover:SetPoint("CENTER", AleaUI.UIParent, "CENTER", 0,0)

local alerts_frames = {}
local alerts_frames_used = {}

function AM:SortAlertFrames()
	for i,frame in ipairs(alerts_frames_used) do
		frame:ClearAllPoints()
	end

	for i,frame in ipairs(alerts_frames_used) do
		if E.db.alerts.growUp then
			frame:SetPoint("BOTTOM", mover, "TOP", 0, 25*(i-1))
		else
			frame:SetPoint("TOP", mover, "BOTTOM", 0, -25*(i-1))
		end
	end
end

function AM:UpdateSettings()
	AM:SortAlertFrames()
	
	for i, frame in ipairs(alerts_frames) do
		frame.background:SetTexture(E:GetTexture(E.db.alerts.texture))
	end
	
	for i, frame in ipairs(alerts_frames_used) do
		frame.background:SetTexture(E:GetTexture(E.db.alerts.texture))
	end
	
end

local function OnShow_Tolltip(self)
	
	if self.tolltip then
	
		gametooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
		gametooltip:SetHyperlink(self.tolltip)
		
		gametooltip:Show()

	elseif self.item_tolltip then
		
		gametooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
		gametooltip:SetHyperlink(self.item_tolltip)
		
		gametooltip:Show()
	
	elseif self.followerID then
	
		local link = C_Garrison.GetFollowerLink(self.followerID);
		if ( link ) then
			GarrisonFollowerTooltip:ClearAllPoints();
			GarrisonFollowerTooltip:SetPoint("BOTTOM", self, "TOP");
			
			GarrisonShipyardFollowerTooltip:ClearAllPoints()
			GarrisonShipyardFollowerTooltip:SetPoint("BOTTOM", self, "TOP");
			
			local _, garrisonFollowerID, quality, level, itemLevel, ability1, ability2, ability3, ability4, trait1, trait2, trait3, trait4 = strsplit(":", link);
			GarrisonFollowerTooltip_Show(tonumber(garrisonFollowerID), false, tonumber(quality), tonumber(level), 0, 0, tonumber(itemLevel), tonumber(ability1), tonumber(ability2), tonumber(ability3), tonumber(ability4), tonumber(trait1), tonumber(trait2), tonumber(trait3), tonumber(trait4));
		end
	elseif self.tradeSkillID and self.recipeID then
	
	
	end
end

local missTexture = GetSpellTexture(202)

function ALEAUI_GetAlertFrame(used, tag, text, icon, id, point, ...)

	local f = tremove(alerts_frames) or AM:CreateAlertFrame()
	
	f.elapsed = 0
	f:SetAlpha(1)
	f:Show()
	
	f.tolltip = nil
	f.item_tolltip = nil
	f.followerID = nil
	f.tradeSkillID = nil
	f.recipeID = nil
	
	f.background:SetTexture(E:GetTexture(E.db.alerts.texture))
	f.background:SetVertexColor(unpack(E.db.alerts.colors[used] or colors[used]))
	f.icon.texture:SetTexCoord(0,1,0,1)
	
--	f.text:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size-4, "OUTLINE")
	
	icon = icon or missTexture
	
--	text = used..'-'..tag..'\n'..text
	
	if used == "LFG" then
		f.text:SetText(text)
		f.icon.texture:SetTexture("Interface\\LFGFrame\\LFGIcon-"..icon)
	elseif used == 'QUEST' then
		f.text:SetText(text)
		f.icon.texture:SetTexture('Interface\\GossipFrame\\AvailableQuestIcon')
	elseif used == 'XP' then
		f.text:SetText(text)
		f.icon.texture:SetTexture("Interface\\Icons\\xp_icon")
		f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
	elseif used == "ITEM" then
		
		f.text:SetText(text)
		f.icon.texture:SetTexture(icon)
		f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
		f.item_tolltip = id
	elseif used == "MONEY" then
		
		f.text:SetText(text)
		f.icon.texture:SetTexture(icon)
		f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
		
	elseif used == 'RESORCES' then
	
		f.text:SetText(text)
		f.icon.texture:SetTexture(icon)
		f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
	
	elseif used == "GARRISON" or used == "GARRISON_BUILD" then
	
		local filename, width, height, left, right, top, bottom, tilesHoriz, tilesVert = GetAtlasInfo(icon)
	
	
		f.text:SetText(text)
		
		f.icon.texture:SetTexture(filename)
		f.icon.texture:SetTexCoord(left, right, top, bottom)
	elseif used == "GARRISON_FOLLOW" then
		local filename, width, height, left, right, top, bottom, tilesHoriz, tilesVert = GetAtlasInfo(icon)
	
	
		f.text:SetText(text)
		
		f.icon.texture:SetTexture(filename)
		f.icon.texture:SetTexCoord(left, right, top, bottom)
		f.followerID = id
	elseif used == "ACH" or used == "CRITERIA" then
		if point and point > 0 then
			f.text:SetText("("..point..") "..text)
		else
			f.text:SetText(text)
		end	
		f.tolltip = GetAchievementLink(id)
		f.icon.texture:SetTexture(icon)
		f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
	elseif used == "NEW_RECIPE" then
		
		f.text:SetText(text)
		f.icon.texture:SetTexture(icon)
		f.tradeSkillID = id
		f.recipeID = point
	else
		
		local itemname, itemlink = GetItemInfo(64998)
		f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
		f.item_tolltip = itemlink
		f.icon.texture:SetTexture("Interface\\Icons\\spell_shadow_shadowwordpain")
		
		if text then
			f.text:SetText(text)
		else
			f.text:SetText("TEST ALERT "..( itemname or " NO NAME"))
		end
	end

	
	table.insert(alerts_frames_used, f)
	AM:SortAlertFrames()
end

function AM:RemoveAlertFrame(frame1)	
	for i,frame in ipairs(alerts_frames_used) do
	
		if frame == frame1 then
			table.insert(alerts_frames, table.remove(alerts_frames_used, i))
			return AM:SortAlertFrames()			
		end
	end
end


function AM:CreateAlertFrame()
	
	local f = CreateFrame("Frame", nil, AleaUI.UIParent)
	f:SetSize(350, 20)
	
	f:SetFrameStrata("HIGH")
	
	f.background = f:CreateTexture(nil, "BACKGROUND")

	f.background:SetPoint("TOPLEFT", f, "TOPLEFT", 23, 0)
	f.background:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)
	f.background:SetTexture(E:GetTexture(E.db.alerts.texture))
	f.background:SetColorTexture(0.2, 0.2, 0.2, 0.7)
	
	f.icon = CreateFrame("Frame", nil, f)
	f.icon:SetPoint("TOPLEFT", f, "TOPLEFT", -1, 0)
	f.icon:SetSize(20,20)
	f.icon.f = f

	E:CreateBackdrop(f.icon, nil, {0,0,0,1}, {0.2,0.2,0.2,1})
	
	f.icon.texture = f.icon:CreateTexture(nil, "ARTWORK")
	f.icon.texture:SetTexCoord(unpack(AleaUI.media.texCoord))
	f.icon.texture:SetAllPoints()
	f.icon.texture:SetTexture("Interface\\Icons\\spell_shadow_shadowwordpain")
	
	f.text = f:CreateFontString(nil, "OVERLAY")
	f.text:SetFontObject(GameFontWhite)
	f.text:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size, "OUTLINE")
	f.text:SetSize(1,1)
	f.text:SetJustifyH("LEFT")
	f.text:SetJustifyV('CENTER')
	f.text:SetPoint("TOPLEFT", f.background, "TOPLEFT", 2,-2)
	f.text:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", -2,2)
	f.text:SetText("TEST")
	f.text:SetWordWrap(true)
	f.text:SetMaxLines(2)
	
	E:CreateBackdrop(f, f.background, {0,0,0,1}, {0,0,0,0})

	f.elapsed = 0
	
	f:SetScript("OnUpdate", function(self, elapsed)
	
		if self.ismouseover then return end
		
		self.elapsed = self.elapsed + elapsed
		
		if self.elapsed > E.db.alerts.fadeTime then
			
			local alpha = self:GetAlpha()-elapsed
	
			if alpha <= 0 then
				self:Hide()
				self:SetAlpha(1)		
				self.elapsed = 0			
				AM:RemoveAlertFrame(self)
			else
				self:SetAlpha(self:GetAlpha()-elapsed)
			end
		else
			self:SetAlpha(1)
		end
	end)
	
	f:SetScript("OnEnter", function(self)
		self.elapsed = 0
		self:SetAlpha(1)
		
		OnShow_Tolltip(self)
		
		self.ismouseover = true
	end)
	f:SetScript("OnShow", function(self)
		self.elapsed = 0
		self:SetAlpha(1)

	end)
	f:SetScript("OnLeave", function(self)
		self.elapsed = 0
		self:SetAlpha(1)
		
		gametooltip:Hide()
		GarrisonFollowerTooltip:Hide()
		GarrisonShipyardFollowerTooltip:Hide()		
		self.ismouseover = false
	end)
	
	f.icon:SetScript("OnEnter", function(self)
		self.f.elapsed = 0
		self.f:SetAlpha(1)
		
		OnShow_Tolltip(self.f)
		self.f.ismouseover = true
	end)

	f.icon:SetScript("OnLeave", function(self)
		self.f.elapsed = 0
		self.f:SetAlpha(1)
		gametooltip:Hide()
		GarrisonFollowerTooltip:Hide()
		GarrisonShipyardFollowerTooltip:Hide()		
		self.f.ismouseover = false
	end)
	
	return f
end

function AM:SHOW_PVP_FACTION_LOOT_TOAST(event, ...)
	local typeIdentifier, itemLink, quantity, specID, sex, isPersonal, lessAwesome = ...;
	if ( typeIdentifier == "item" ) then
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
		local color = ItemQualityTable[itemRarity];
		local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
	
		ALEAUI_GetAlertFrame("ITEM", 'PVP1', E.RGBToHex(r*255, g*255, b*255)..itemName, itemTexture, itemLink, 0)
	elseif ( typeIdentifier == "money" ) then
	--	MoneyWonAlertFrame_ShowAlert(quantity);
		
		ALEAUI_GetAlertFrame("MONEY", 'PVP1', E.RGBToHex(237, 183, 19)..GetMoneyString(quantity), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
	elseif ( typeIdentifier == "currency" ) then	
		local name, amount, texturePath, earnedThisWeek, weeklyMax, totalMax, isDiscovered = GetCurrencyInfo(itemLink)
		ALEAUI_GetAlertFrame("RESORCES", 'PVP1', E.RGBToHex(255, 255, 255)..name..'('..quantity..')', texturePath, 0, 0)
	end
end

function AM:ACHIEVEMENT_EARNED(event, achievementID, alreadyEarned)
	
	local _, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch, wasEarnedByMe, earnedBy = GetAchievementInfo(achievementID);
	
--	print(GetAchievementInfo(achievementID))
	
	ALEAUI_GetAlertFrame("ACH", 'ACHIVEMENT1', name, icon, achievementID, points)
end

function AM:CRITERIA_EARNED(event, achievementID, criteriaID)
	
	if criteriaID and type(criteriaID) == 'string' then
		local _, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch, wasEarnedByMe, earnedBy = GetAchievementInfo(achievementID);
		
		ALEAUI_GetAlertFrame("CRITERIA", 'CRITERIA1', criteriaID, icon, achievementID, 0)
		
	else
		local criteriaString, criteriaType, completed, quantity, reqQuantity, charName, flags, assetID, quantityString, criteriaID, eligible =  GetAchievementCriteriaInfoByID(achievementID, criteriaID);
		local _, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuildAch, wasEarnedByMe, earnedBy = GetAchievementInfo(achievementID);
		
	--	print(GetAchievementCriteriaInfoByID(achievementID, criteriaID))
		
		ALEAUI_GetAlertFrame("CRITERIA", 'CRITERIA2', criteriaString.." ("..quantity.."/"..reqQuantity..")", icon, achievementID, 0)
	
	end
end

function AM:LFG_COMPLETION_REWARD(...)
	local name, typeID, subtypeID, textureFilename, moneyBase, moneyVar, experienceBase, experienceVar, numStrangers, numRewards= GetLFGCompletionReward();
	local alertName = name
	
	local moneyAmount = moneyBase + moneyVar * numStrangers;	
	local experienceGained = experienceBase + experienceVar * numStrangers;
	
	if ( C_Scenario.IsInScenario() ) then
		local _, _, _, _, hasBonusStep, isBonusStepComplete = C_Scenario.GetInfo();
	
		if ( hasBonusStep and isBonusStepComplete ) then
			alertName = alertName.." "..SCENARIO_BONUS_LABEL
		end

		if moneyAmount > 0 then
			ALEAUI_GetAlertFrame("MONEY", 'LFG-Complete1', E.RGBToHex(237, 183, 19)..GetMoneyString(moneyAmount), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
		end	
		if experienceGained > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
			ALEAUI_GetAlertFrame("XP",  'LFG-Complete1', E.RGBToHex(255, 255, 255)..XP..":"..E.NumCompress(experienceGained))
		end
		
		ALEAUI_GetAlertFrame("LFG",  'LFG-Complete1', alertName, textureFilename, 0, 0)
		
	else
		if moneyAmount > 0 then
			ALEAUI_GetAlertFrame("MONEY",  'LFG-Complete2', E.RGBToHex(237, 183, 19)..GetMoneyString(moneyAmount), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
		end
		if experienceGained > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
			ALEAUI_GetAlertFrame("XP",  'LFG-Complete2', E.RGBToHex(255, 255, 255)..XP..":"..E.NumCompress(experienceGained))
		end
		
		ALEAUI_GetAlertFrame("LFG",  'LFG-Complete2', alertName, textureFilename, 0, 0)		
	end
end

local GCC_tex = "Interface\\Icons\\achievement_bg_wineos_underxminutes"
function AM:GUILD_CHALLENGE_COMPLETED(event, ...)
	local challengeType, count, max1 = ...
	ALEAUI_GetAlertFrame("MONEY", 'GUILD-Challenge-Complete1', format("%s %s %d/%d", "Гильдейские испытания", _G["GUILD_CHALLENGE_TYPE"..challengeType], count, max1) , GCC_tex, 0,0)	
end

function AM:CHALLENGE_MODE_COMPLETED(...)
	
	local mapID, medal, completionTime, moneyAmount, numRewards = GetChallengeModeCompletionInfo();
	local medal_name = ""
	
	if ( CHALLENGE_MEDAL_TEXTURES[medal] ) then
		texture = CHALLENGE_MEDAL_TEXTURES[medal]
	end
	
	if medal == 1 then medal_name = "M1"
	elseif medal == 2 then medal_name = "M2"
	elseif medal == 3 then medal_name = "M3"
	elseif medal == 4 then medal_name = "M4"
	end
	
	local money = ""
	
	if ( moneyAmount > 0 ) then
		money = " Награда:"..GetMoneyString(moneyAmount)
	end
	
	ALEAUI_GetAlertFrame("MONEY", 'Challenge-Complete1', medal..money, GCC_tex, 0,0)
	
end

function AM:LOOT_ITEM_ROLL_WON(event, itemLink, quantity, rollType, roll)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
	local color = ItemQualityTable[itemRarity];
	local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1

	ALEAUI_GetAlertFrame("ITEM", 'LootItemRollWon', E.RGBToHex(r*255, g*255, b*255)..itemName, itemTexture, itemLink, 0)
end
function AM:SHOW_LOOT_TOAST(event, typeIdentifier, itemLink, quantity, _4, _5, _6, source)

	if ( typeIdentifier == "item" ) then
		
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
		local color = ItemQualityTable[itemRarity];
		local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
	
		ALEAUI_GetAlertFrame("ITEM", 'ShowLootToast1', E.RGBToHex(r*255, g*255, b*255)..itemName, itemTexture, itemLink, 0)
		
	elseif ( typeIdentifier == "money" ) then
		
		ALEAUI_GetAlertFrame("MONEY", 'ShowLootToast2', E.RGBToHex(237, 183, 19)..GetMoneyString(quantity), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
		
	elseif ( typeIdentifier == "currency" and source == 10 and itemLink:match("currency:824") ) then
		local name, amount, texturePath, earnedThisWeek, weeklyMax, totalMax, isDiscovered = GetCurrencyInfo(824)
		
		ALEAUI_GetAlertFrame("RESORCES", 'ShowLootToast3', E.RGBToHex(255, 255, 255)..name..'('..quantity..')', texturePath, 0, 0)
	end
end

function AM:PET_BATTLE_CLOSE(...)

end

function AM:STORE_PRODUCT_DELIVERED(event, icon)	
	ALEAUI_GetAlertFrame("MONEY", 'StoreDelivered1', "Предмет доставлен", icon, 0, 0)
end

function AM:SHOW_LOOT_TOAST_UPGRADE(event, ...)	
	local itemLink, quantity, specID, sex, baseQuality, isPersonal = ...;
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
	local color = ItemQualityTable[itemRarity];
	local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
	ALEAUI_GetAlertFrame("ITEM", 'LootToastUpgrade1', E.RGBToHex(r*255, g*255, b*255)..itemName.."("..tostring(itemLevel)..")" , itemTexture, itemLink, 0)
end

function AM:GARRISON_BUILDING_ACTIVATABLE(event, name)
	ALEAUI_GetAlertFrame("GARRISON_BUILD", 'GarrisonBuild1', "Постройка "..name.." завершена.", "GarrMission_MissionIcon-Combat", 0, 0)
end

function AM:GARRISON_MISSION_FINISHED(event, ...)
	local missionInfo
	
	if E.IsLegion then
		local missionIndex, missionID = ...
		missionInfo = C_Garrison.GetBasicMissionInfo(missionID);
	else
		local missionID = ...
		missionInfo = C_Garrison.GetBasicMissionInfo(missionID);
	end
	
	ALEAUI_GetAlertFrame("GARRISON", 'GarrisonMissionFinished1', 'Миссия "'..missionInfo.name..'" завершена.', missionInfo.typeAtlas, 0, 0)
end

function AM:GARRISON_FOLLOWER_ADDED(event, ...)
	local followerID, name, class, displayID, level, quality, isUpgraded, texPrefix, followerType = ...;
	local color = ITEM_QUALITY_COLORS[quality] or { r = 0, g = 1, b = 0 };
	
	if (followerType == LE_FOLLOWER_TYPE_GARRISON_6_0) then
		ALEAUI_GetAlertFrame("GARRISON_FOLLOW", 'GarrisonFollowerAdded1', AleaUI.RGBToHex(color.r*255, color.g*255, color.b*255)..'Новый соратник "'..name..'"', "GarrMission_MissionIcon-Combat", followerID, 0)
	else
		ALEAUI_GetAlertFrame("GARRISON_FOLLOW", 'GarrisonFollowerAdded2',AleaUI.RGBToHex(color.r*255, color.g*255, color.b*255)..'Новый корабль "'..name..'"', "GarrMission_MissionIcon-Combat", followerID, 0)
	end
end

function AM:NEW_RECIPE_LEARNED(event, recipeID)
	local tradeSkillID, skillLineName = C_TradeSkillUI.GetTradeSkillLineForRecipe(recipeID);
	if tradeSkillID then
		local recipeName = GetSpellInfo(recipeID);
		if recipeName then

			local rank = GetSpellRank(recipeID);

			local name = rank and rank > 1 and UPGRADED_RECIPE_LEARNED_TITLE or NEW_RECIPE_LEARNED_TITLE

			local rankTexture = NewRecipeLearnedAlertFrame_GetStarTextureFromRank(rank);
			if rankTexture then
				name = name..' '..format("%s %s", recipeName, rankTexture)
			else
				name = name..' '..recipeName
			end

			ALEAUI_GetAlertFrame('NEW_RECIPE', 'NewRecipeLearned1', name, C_TradeSkillUI.GetTradeSkillTexture(tradeSkillID), tradeSkillID, recipeID)
		end
	end

end

function AM:SHOW_LOOT_TOAST_LEGENDARY_LOOTED(event, ...)
	local itemLink = ...;

	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
	local color = ItemQualityTable[itemRarity];
	local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1

	ALEAUI_GetAlertFrame("ITEM", 'LegendaryItem1', E.RGBToHex(r*255, g*255, b*255)..itemName, itemTexture, itemLink, 0)
end
	
function AM:SCENARIO_COMPLETED(event, ...)
	
	local scenarioName, currentStage, numStages, flags, hasBonusStep, isBonusStepComplete, _, xp, money, scenarioType, areaName = C_Scenario.GetInfo();
	
	if scenarioType == LE_SCENARIO_TYPE_LEGION_INVASION then
		
		if true then return end
		
		local alertName = 'Задание завершено'
		
		if scenarioName then
			alertName = 'Выполнено "'..scenarioName..'"'
		elseif areaName then
			alertName = 'Выполнено "'..areaName..'"'
		end
		
		if money > 0 then
			ALEAUI_GetAlertFrame("MONEY", 'SCENARIO_COMPLETED1',E.RGBToHex(237, 183, 19)..GetMoneyString(money), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
		end
		if xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
			ALEAUI_GetAlertFrame("XP", 'SCENARIO_COMPLETED1',E.RGBToHex(255, 255, 255)..XP..":"..E.NumCompress(xp))
		end
			
			
		if rewardItemLink then
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
			local color = itemRarity and ItemQualityTable[itemRarity] or false
			local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
			
			local _, _, _, _, textureInstant = GetItemInfoInstant(rewardItemLink);
			
			ALEAUI_GetAlertFrame("ITEM", 'SCENARIO_COMPLETED1',E.RGBToHex(r*255, g*255, b*255)..(itemName or 'Предмет получен'), itemTexture or textureInstant, rewardItemLink, 0)
		end
		
		ALEAUI_GetAlertFrame('QUEST', 'SCENARIO_COMPLETED1', alertName)
	else
		local alertName = 'Сценарий завершен'
		if scenarioName then
			alertName = 'Выполнено "'..scenarioName..'"'
		elseif areaName then
			alertName = 'Выполнено "'..areaName..'"'
		end
		
		ALEAUI_GetAlertFrame('QUEST', 'SCENARIO_COMPLETED2', alertName)
	end
end	

function AM:QUEST_LOOT_RECEIVED(event, ...)
	local questID, rewardItemLink = ...;
	
	if QuestMapFrame_IsQuestWorldQuest(questID) then
	--	WorldQuestCompleteAlertSystem:AddAlert(questID, rewardItemLink);	
		local isInArea, isOnMap, numObjectives, taskName, displayAsObjective = GetTaskInfo(questID);
		local icon = WorldQuestCompleteAlertFrame_GetIconForQuestID(questID);
		local money = GetQuestLogRewardMoney(questID);
		local xp = GetQuestLogRewardXP(questID);
		
		local alertName = taskName and 'Выполнено "'..taskName..':' or 'Задание завершено'

		if money > 0 then
			ALEAUI_GetAlertFrame("MONEY", 'QUEST_LOOT_RECEIVED1', E.RGBToHex(237, 183, 19)..GetMoneyString(money), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
		end
		if xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
			ALEAUI_GetAlertFrame("XP", 'QUEST_LOOT_RECEIVED1', E.RGBToHex(255, 255, 255)..XP..":"..E.NumCompress(xp))
		end
		for currencyIndex = 1, GetNumQuestLogRewardCurrencies(questID) do
			local name, texture, count = GetQuestLogRewardCurrencyInfo(currencyIndex, questID);

			ALEAUI_GetAlertFrame("RESORCES", 'QUEST_LOOT_RECEIVED1', E.RGBToHex(255, 255, 255)..name..'('..count..')', texture, 0, 0)			
		end
		
		if rewardItemLink then
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);
			local color = itemRarity and ItemQualityTable[itemRarity] or false
			local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
			
			local _, _, _, _, textureInstant = GetItemInfoInstant(rewardItemLink);
			
			ALEAUI_GetAlertFrame("ITEM", 'QUEST_LOOT_RECEIVED1', E.RGBToHex(r*255, g*255, b*255)..(itemName or 'Предмет получен'), itemTexture or textureInstant, rewardItemLink, 0)
		end
	
		ALEAUI_GetAlertFrame('QUEST', 'QUEST_LOOT_RECEIVED1', alertName)
	else
		local scenarioName, currentStage, numStages, flags, hasBonusStep, isBonusStepComplete, _, xp, money, scenarioType, areaName = C_Scenario.GetInfo();
		local isInArea, isOnMap, numObjectives, taskName, displayAsObjective = GetTaskInfo(questID);
		
		
		local alertName = 'Задание завершено'
		
	--	print('QUEST_LOOT_RECEIVED', 'QuestID', questID, 'TaskName', taskName)
		
		if scenarioName then
			alertName = 'Выполнено "'..scenarioName..'"'
		elseif areaName then
			alertName = 'Выполнено "'..areaName..'"'
		end
	
		if money > 0 then
			ALEAUI_GetAlertFrame("MONEY", 'QUEST_LOOT_RECEIVED2', E.RGBToHex(237, 183, 19)..GetMoneyString(money), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
		end
		if xp > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL then
			ALEAUI_GetAlertFrame("XP", 'QUEST_LOOT_RECEIVED2', E.RGBToHex(255, 255, 255)..XP..":"..E.NumCompress(xp))
		end
			
			
		if rewardItemLink then
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(rewardItemLink);
			local color = itemRarity and ItemQualityTable[itemRarity] or false
			local r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
			
			local _, _, _, _, textureInstant = GetItemInfoInstant(rewardItemLink);
			
			ALEAUI_GetAlertFrame("ITEM", 'QUEST_LOOT_RECEIVED2', E.RGBToHex(r*255, g*255, b*255)..(itemName or 'Предмет получен'), itemTexture or textureInstant, rewardItemLink, 0)
		end
		
		ALEAUI_GetAlertFrame('QUEST', 'QUEST_LOOT_RECEIVED2', alertName)
	end
end	

function AM:GARRISON_RANDOM_MISSION_ADDED(event, missionID)
	local missionInfo = C_Garrison.GetBasicMissionInfo(missionID);
	local level = missionInfo.level
	local iLevel = "(" .. missionInfo.iLevel .. ")"
	local isRare = missionInfo.isRare
	local name = missionInfo.name
	
	ALEAUI_GetAlertFrame('QUEST', 'GARRISON_RANDOM_MISSION_ADDED1', 'Выполнено "'..name..'"')
end

AM:Register(events)

local function InitAlertMover()
	E:Mover(mover, "alertMover")

	
--	ScenarioAlertFrame1.Show = ScenarioAlertFrame1.Hide
--	ScenarioAlertFrame1:Hide()
	AM:UpdateSettings()
	
	AlertFrame:UnregisterEvent("ACHIEVEMENT_EARNED")
	AlertFrame:UnregisterEvent("CRITERIA_EARNED")
	AlertFrame:UnregisterEvent("LFG_COMPLETION_REWARD")
	AlertFrame:UnregisterEvent("GUILD_CHALLENGE_COMPLETED")
	AlertFrame:UnregisterEvent("CHALLENGE_MODE_COMPLETED")
	AlertFrame:UnregisterEvent("LOOT_ITEM_ROLL_WON")
	AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST")
	AlertFrame:UnregisterEvent("STORE_PRODUCT_DELIVERED")
	AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST_UPGRADE")
	
	AlertFrame:UnregisterEvent("SHOW_PVP_FACTION_LOOT_TOAST")
	
	AlertFrame:UnregisterEvent("GARRISON_BUILDING_ACTIVATABLE")
	AlertFrame:UnregisterEvent("GARRISON_MISSION_FINISHED")
	AlertFrame:UnregisterEvent("GARRISON_FOLLOWER_ADDED")
	
	AlertFrame:UnregisterEvent("NEW_RECIPE_LEARNED")
	AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST_LEGENDARY_LOOTED")
	
	AlertFrame:UnregisterEvent("SCENARIO_COMPLETED")
	AlertFrame:UnregisterEvent("QUEST_LOOT_RECEIVED")
	
	AlertFrame:UnregisterEvent("GARRISON_RANDOM_MISSION_ADDED")
	
end

if AlertFrameMixin then
	hooksecurefunc(AlertFrameMixin, "OnLoad", function(self)

		self:UnregisterEvent("ACHIEVEMENT_EARNED")
		self:UnregisterEvent("CRITERIA_EARNED")
		self:UnregisterEvent("LFG_COMPLETION_REWARD")
		self:UnregisterEvent("GUILD_CHALLENGE_COMPLETED")
		self:UnregisterEvent("CHALLENGE_MODE_COMPLETED")
		self:UnregisterEvent("LOOT_ITEM_ROLL_WON")
		self:UnregisterEvent("SHOW_LOOT_TOAST")
		self:UnregisterEvent("STORE_PRODUCT_DELIVERED")
		self:UnregisterEvent("SHOW_LOOT_TOAST_UPGRADE")
		
		self:UnregisterEvent("SHOW_PVP_FACTION_LOOT_TOAST")
		
		self:UnregisterEvent("GARRISON_BUILDING_ACTIVATABLE")
		self:UnregisterEvent("GARRISON_MISSION_FINISHED")
		self:UnregisterEvent("GARRISON_FOLLOWER_ADDED")
		
		self:UnregisterEvent("NEW_RECIPE_LEARNED")
		self:UnregisterEvent("SHOW_LOOT_TOAST_LEGENDARY_LOOTED")

		self:UnregisterEvent("SCENARIO_COMPLETED")
		self:UnregisterEvent("QUEST_LOOT_RECEIVED")
		
		self:UnregisterEvent("GARRISON_RANDOM_MISSION_ADDED")
		
		self:HookScript('OnEvent', function(...)
			print(...)
		end)
	end)
else
	hooksecurefunc("AlertFrame_OnLoad", function(self)

		AlertFrame:UnregisterEvent("ACHIEVEMENT_EARNED")
		AlertFrame:UnregisterEvent("CRITERIA_EARNED")
		AlertFrame:UnregisterEvent("LFG_COMPLETION_REWARD")
		AlertFrame:UnregisterEvent("GUILD_CHALLENGE_COMPLETED")
		AlertFrame:UnregisterEvent("CHALLENGE_MODE_COMPLETED")
		AlertFrame:UnregisterEvent("LOOT_ITEM_ROLL_WON")
		AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST")
		AlertFrame:UnregisterEvent("STORE_PRODUCT_DELIVERED")
		AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST_UPGRADE")
		
		AlertFrame:UnregisterEvent("SHOW_PVP_FACTION_LOOT_TOAST")
		
		AlertFrame:UnregisterEvent("GARRISON_BUILDING_ACTIVATABLE")
		AlertFrame:UnregisterEvent("GARRISON_MISSION_FINISHED")
		AlertFrame:UnregisterEvent("GARRISON_FOLLOWER_ADDED")
		
		AlertFrame:UnregisterEvent("NEW_RECIPE_LEARNED")
		AlertFrame:UnregisterEvent("SHOW_LOOT_TOAST_LEGENDARY_LOOTED")
		
		AlertFrame:UnregisterEvent("SCENARIO_COMPLETED")
		AlertFrame:UnregisterEvent("QUEST_LOOT_RECEIVED")
		
		AlertFrame:UnregisterEvent("GARRISON_RANDOM_MISSION_ADDED")
	
	
	end)
end

BonusRollMoneyWonFrame:Kill()
hooksecurefunc('MoneyWonAlertFrame_SetUp', function(self, rewardQuantity)
	self:Hide()
	self.animIn:Stop()
	ALEAUI_GetAlertFrame("MONEY", nil, E.RGBToHex(237, 183, 19)..GetMoneyString(rewardQuantity), "Interface\\Icons\\INV_Misc_Coin_02", 0, 0)
	AlertFrame:UpdateAnchors();
end)

BonusRollLootWonFrame:Kill()
hooksecurefunc('LootWonAlertFrame_SetUp', function(self, rewardLink, rewardQuantity, rollType, roll, rewardSpecID, isCurrency, showFactionBG, lootSource, lessAwesome, isUpgraded)
	self:Hide()
	self.animIn:Stop()
	
	local itemName, itemHyperLink, itemRarity, itemTexture, itemLevel, _
	local color;
	local r, g, b
	
	if (isCurrency == true) then
		itemName, _, itemTexture, _, _, _, _, itemRarity = GetCurrencyInfo(rewardLink);
		if ( lootSource == LOOT_SOURCE_GARRISON_CACHE ) then
			itemName = format(GARRISON_RESOURCES_LOOT, rewardQuantity);
		else
			itemName = format(CURRENCY_QUANTITY_TEMPLATE, rewardQuantity, itemName);
		end
		itemHyperLink = rewardLink;		
	else
		itemName, itemHyperLink, itemRarity, itemLevel, _, _, _, _, _, itemTexture = GetItemInfo(rewardLink);
	end
	
	color = ItemQualityTable[itemRarity];
	r, g, b = color and color.r or 1, color and color.g or 1, color and color.b or 1
	
	if not itemName then 
		AlertFrame:UpdateAnchors();
		return 
	end
	
	ALEAUI_GetAlertFrame("ITEM", nil, E.RGBToHex(r*255, g*255, b*255)..itemName.."("..tostring(itemLevel or 'NoiLvl')..")" , itemTexture, rewardLink, 0)
	
	AlertFrame:UpdateAnchors();
end)


E:OnInit2(InitAlertMover)

E.GUI.args.general.args.alerts = {
	name = 'Уведомления',
	order = 5,
	type = 'group',
	embend = true,
	args = {},
}

E.GUI.args.general.args.alerts.args.growUp = {
	name = 'Рост вверх',
	order = 1,
	width = 'full',
	type = 'toggle',
	set = function()
		E.db.alerts.growUp = not E.db.alerts.growUp
		
		AM:UpdateSettings()
	end,
	get = function()
		return E.db.alerts.growUp
	end,
}

E.GUI.args.general.args.alerts.args.showInCombat = {
	name = 'Показывать в бою',
	order = 2,
	width = 'full',
	type = 'toggle',
	set = function()
		E.db.alerts.showInCombat = not E.db.alerts.showInCombat
	end,
	get = function()
		return E.db.alerts.showInCombat
	end,
}

E.GUI.args.general.args.alerts.args.texture = {
	name = 'Текстура',
	order = 3,
	type = 'statusbar',
	values = E.GetTextureList,
	set = function(info, value)
		E.db.alerts.texture = value
		
		AM:UpdateSettings()
	end,
	get = function()
		return E.db.alerts.texture
	end,
}

E.GUI.args.general.args.alerts.args.fadeTime = {
	name = 'Время исчезновения',
	order = 4,
	type = 'slider',
	min = 4, max = 40, step = 1,
	set = function(info, value)
		E.db.alerts.fadeTime = value
	end,
	get = function(info)
		return E.db.alerts.fadeTime
	end,
}

E.GUI.args.general.args.alerts.args.colors = {
	name = 'Цвета',
	order = 5,
	type = 'group',
	embend = true,
	args = {},
}
for k,v in pairs(alertsName) do	
	E.GUI.args.general.args.alerts.args.colors.args[k] = {
		name = v,
		order = 1,
		type = "color",
		set = function(self, r,g,b)
			E.db.alerts.colors[k] = { r, g, b, 1 }
		end,
		get = function(self)
			return E.db.alerts.colors[k][1], E.db.alerts.colors[k][2], E.db.alerts.colors[k][3], 1
		end,
	}	
end